import dao.*
import io.javalin.Javalin
import io.javalin.apibuilder.ApiBuilder.*
import io.javalin.core.security.Role
import io.javalin.core.security.SecurityUtil.roles
import org.jetbrains.exposed.sql.Database
import org.jetbrains.exposed.sql.SchemaUtils
import org.jetbrains.exposed.sql.StdOutSqlLogger
import org.jetbrains.exposed.sql.addLogger
import org.jetbrains.exposed.sql.transactions.transaction

enum class ApiRole : Role { PUBLIC, BASIC_AUTHED, API_AUTHED }

fun main() {
    val listIdStr = "list-id"
    var migrationsController = MigrationsController()
    val app = Javalin.create {
        it.accessManager { handler, ctx, permittedRoles ->
            transaction {
                val maybeKey = Key.find {
                    Keys.value eq ctx.header("X-API-KEY").orEmpty()
                }.firstOrNull()
                val auth = ctx.header("Authorization").orEmpty()
                val isBasicAuthed = auth == System.getenv("LDC_AUTH")
                when {
                    permittedRoles.contains(ApiRole.PUBLIC) -> handler.handle(ctx)
                    permittedRoles.contains(ApiRole.BASIC_AUTHED) && isBasicAuthed -> handler.handle(ctx)
                    permittedRoles.contains(ApiRole.API_AUTHED) && maybeKey != null -> handler.handle(ctx)
                    else -> ctx.status(401).json("Unauthorized")
                }
            }
        }
    }.start(7000)
    Database.connect("jdbc:sqlite:data.db", "org.sqlite.JDBC")
    transaction {
        addLogger(StdOutSqlLogger)
        SchemaUtils.create (Lists)
        SchemaUtils.create (Items)
        SchemaUtils.create (Keys)
        SchemaUtils.create (Migrations)
        migrationsController.checkMigrationsToDo()

    }

    app.routes {
        path("api") {
            crud("keys/:key-id", KeysController(), roles(ApiRole.BASIC_AUTHED))
            crud("lists/:list-id", ListsController(), roles(ApiRole.API_AUTHED))
        }

        post("/api/keys:check", { ctx ->
            KeysController().checkKey(ctx)
        }, roles(ApiRole.PUBLIC))

        get("/api/lists/:list-id/items", { ctx ->
            ListItemsController().getListItems(ctx, ctx.pathParam(listIdStr))
        }, roles(ApiRole.API_AUTHED))

        post("/api/lists/:list-id/items", { ctx ->
            ListItemsController().create(ctx.pathParam(listIdStr), ctx)
        }, roles(ApiRole.API_AUTHED))

        delete("/api/lists/:list-id/items/:item-id", { ctx ->
            ListItemsController().delete(ctx.pathParam(listIdStr), ctx.pathParam("item-id"), ctx)
        }, roles(ApiRole.API_AUTHED))

        patch("/api/lists/:list-id/items/:item-id", { ctx ->
            ListItemsController().update(ctx.pathParam(listIdStr), ctx.pathParam("item-id"), ctx)
        }, roles(ApiRole.API_AUTHED))
    }
}